/* REXX
  
   Prints info on all smb responding machines on a subnet.
   This script needs to be run on a machine without nmbd running and be
   run as root to get correct info from WIN95 clients.
  
   syntax:
      findsmb [-d|-D] [-r] [subnet broadcast address]
  
   with no agrument it will list machines on the current subnet
  
   There will be a "+" in front of the workgroup name for machines that are
   local master browsers for that workgroup. There will be an "*" in front
   of the workgroup name for machines that are the domain master browser for
   that workgroup.
   
   Options:
  
   -d|-D		enable debug
   -r		add -r option to nmblookup when finding netbios name
 */

'@echo off'
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs
        
parse source . . sambabin

sambabin = filespec('D',sambabin)||strip(filespec('P',sambabin),'T','\')

parse arg OrgCmdLine
CmdLine = translate(OrgCmdLine)

R_Option = ""
Debug = 0
BCast = ""
Master= " "

nmblookup = value("TEMP",,"OS2ENVIRONMENT")'\nmblookup.txt'
smb       = value("TEMP",,"OS2ENVIRONMENT")'\smb.txt'

/* test for -d and -r options and
   set broadcast address if it was specified */

do i = 1 to words(Cmdline)
    if word(CmdLine,i) = '-R'| word(CmdLine,i) = '/R' then R_Option = "-r"
    if word(CmdLine,i) = '-D'| word(CmdLine,i) = '/D' then Debug    = 1
    if word(CmdLine,i) = '-B'| word(CmdLine,i) = '/B' then BCast    = "-B "word(CmdLine,I+1) /* error check missing here */
end

/* look for all machines that respond to a name lookup */


sambabin"/nmblookup "BCAST" '*' --debuglevel=0 >"nmblookup

/* get rid of all lines that are not a response IP address,
   strip everything but IP address and sort by last field in address */

I = 0
do until lines(nmblookup) = 0
    nmbline = linein(nmblookup)
    if pos("*<00>",nmbline) > 0 then do
        I = I + 1
        ipaddrs.I = word(nmbline,1)
        parse var ipaddrs.I part.1'.'part.2'.'part.3'.'part.4
        do J = 1 to 4
            part.J = right('000'part.J,3)
        end
        ipaddrs.I = part.1'.'part.2'.'part.3'.'part.4
    end
end
ipaddrs.0 = I /* stem is not sorted yet */
ok = SysStemSort("ipaddrs","A","I")
ok = stream(nmblookup,'c','close')
do I = 1 to ipaddrs.0
    parse var ipaddrs.I part.1'.'part.2'.'part.3'.'part.4
    do J = 1 to 4
        part.J = strip(part.J,'B','0')
    end
    ipaddrs.I = part.1'.'part.2'.'part.3'.'part.4
end

/* print header info */
say "                                *=DMB"
say "                                +=LMB"
say "IP ADDR         NETBIOS NAME     WORKGROUP/OS/VERSION "BCast
say copies("-",79)

do I = 1 to ipaddrs.0 /* loop through each IP address found */
    Master    = " "
    Workgroup = "unknown"
    Server    = "unknown"
    IsWin95   = 0

	/* find the netbios names registered by each machine */
    SambaBin"/nmblookup "R_Option" -A "ipaddrs.I" --debuglevel=0 >"nmblookup
    
    do until lines(nmblookup) = 0
        nmbline = linein(nmblookup)
        if pos('<1b>', nmbline) > 0 then Master = "+"
        if pos('<1d>', nmbline) > 0 then Master = "*"
        if pos('<00>', nmbline) > 0 then do /* we have a netbios name */
            if pos('GROUP',nmbline) > 0 then do /* is it a group? */
                if IsWin95 then Workgroup = strip(left(strip(nmbline,,'09'x),15))
            end
            else do
                /* The Netbios name can contain lot of characters also < >
                   and spaces. The follwing cure inside name space but not
                   names starting or ending with spaces */
                name = strip(left(strip(nmbline,,'09'x),15))
                SAMBABIN"\smbclient -L "name" -I "ipaddrs.I" -N --debuglevel=1 2>"smb" 1>NUL"
                smbline = linein(smb)
                ok = stream(smb,'c','close') 
                if Debug then do /* Debug */
                    say copies("=",79)
                    'type 'nmblookup
                    'type 'smb
                    say copies("=",79)
                end
                if pos('OS=[',smbline) = 0 then do /* WIN95 client */
                    /* not properly supported yet */
                    OS="Win 95"
                    IsWin95 = 1
                end
                else do
                    parse var smbline "Domain=["Workgroup"] OS=["OS"] Server=["Server"]"Rest
                    if word(OS,1) = "Windows" then OS = "Win"substr(OS,8)
                    if word(Server,1) = "Windows" then Server = "Win"substr(Server,8)
                end
            end
        end
    end
    ok = stream(nmblookup,'c','close')
    say left(left(ipaddrs.I,16)||left(Name,15)||left(Master,2)||WorkGroup'/'OS'/'Server,79)
end

ok = SysFileDelete(nmblookup)
ok = SysFileDelete(smb)
exit
